/*
  NSString-NirAdditions.m
 
  HebrewService - system services for Hebrew users.

  Copyright (C) 2003-2006 Nir Soffer
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the
  Free Software Foundation; either version 2 of the License, or (at your
  option) any later version.
 
  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details:
  http://www.gnu.org/licenses/gpl.html
*/

#import <CoreFoundation/CoreFoundation.h>
#import "NSString-NirAdditions.h"


@implementation NSString (NSStringNirAdditions)

+ (id)stringWithContentsOfFile:(NSString *)absolutePath
                      encoding:(unsigned)encoding; {
    // Read file into NSData then create string using encoding
    NSData *data = [[NSData alloc] initWithContentsOfFile:absolutePath];
    id instance = [[[self class] alloc] initWithData:data
                                            encoding:encoding];
    [data release];
    return [instance autorelease];
}

- (NSString *)stringByUnwrappingLines;
{
    unsigned maxIndex = [self length];
    unsigned lineStart = 0, lineEnd = 0, contentsEnd = 0;
    NSCharacterSet *whitespace = [NSCharacterSet whitespaceCharacterSet];
    NSMutableString *currentParagraph = [[NSMutableString alloc] init];
    NSMutableString *result = [NSMutableString string];    
    
    while (lineEnd < maxIndex) {
        [self getLineStart:&lineStart 
                       end:&lineEnd 
               contentsEnd:&contentsEnd 
                  forRange:NSMakeRange(lineEnd, 0)];
        
        // Get line and strip it
        NSRange lineRange = NSMakeRange(lineStart, contentsEnd - lineStart);
        NSString *line = [self substringWithRange:lineRange];        
        if ([line length] > 0)
            line = [line stringByTrimmingCharactersInSet:whitespace];
        
        // Handle paragraph separators
        if ([line length] == 0) {
            // ignore multiple paragraph separators
            if ([currentParagraph length] == 0)
                continue;
            
            // Append current paragraph and start new one
            [result appendString:currentParagraph];
            [result appendString:@"\n"];
            [currentParagraph release];
            currentParagraph = [[NSMutableString alloc] init];
            continue;
        }
        
        // Append line with adding spaces between lines
        if ([currentParagraph length] != 0)
            [currentParagraph appendString:@" "];
        [currentParagraph appendString:line];        
    }
    
    // Append last paragraph
    if ([currentParagraph length] > 0) {
        [result appendString:currentParagraph];
        [result appendString:@"\n"];
    }
    [currentParagraph release];
    
    return result;
}

@end

